# Project:  MapServer
# Purpose:  xUnit style Python mapscript tests of clusterObj
# Author:   Seth Girvin
#
# ===========================================================================
# Copyright (c) 2019, Seth Girvin
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ===========================================================================

import unittest

import mapscript


class ClusterObjTestCase(unittest.TestCase):
    def testClusterObjUpdateFromString(self):
        """a cluster can be updated from a string"""
        c = mapscript.clusterObj()
        c.updateFromString("CLUSTER \n MAXDISTANCE 5 \n REGION \n 'rectangle' END")

        assert c.maxdistance == 5
        assert c.region == "rectangle"

        s = c.convertToString()
        assert s == 'CLUSTER\n  MAXDISTANCE 5\n  REGION "rectangle"\nEND # CLUSTER\n'

    def testClusterObjGetSetFilter(self):
        """a cluster filter can be set and read"""
        c = mapscript.clusterObj()
        filter = "[attr1] > 5"
        c.setFilter(filter)
        assert '"{}"'.format(filter) == c.getFilterString()

    def testClusterObjGetSetGroup(self):
        """a cluster filter can be set and read"""
        c = mapscript.clusterObj()
        exp = "100"  # TODO not sure what would be a relevant expression here
        c.setGroup(exp)
        assert '"{}"'.format(exp) == c.getGroupString()


if __name__ == "__main__":
    unittest.main()



# Project:  MapServer
# Purpose:  xUnit style Python mapscript tests of rectObj
# Author:   Sean Gillies, sgillies@frii.com
#
# ===========================================================================
# Copyright (c) 2004, Sean Gillies
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ===========================================================================

import unittest

import mapscript

from .testing import MapPrimitivesTestCase


class RectObjTestCase(MapPrimitivesTestCase):
    def testRectObjConstructorNoArgs(self):
        """a rect can be initialized with no args"""
        r = mapscript.rectObj()
        self.assertAlmostEqual(r.minx, -1.0)
        self.assertAlmostEqual(r.miny, -1.0)
        self.assertAlmostEqual(r.maxx, -1.0)
        self.assertAlmostEqual(r.maxy, -1.0)

    def testRectObjConstructorArgs(self):
        """a rect can be initialized with arguments"""
        r = mapscript.rectObj(-1.0, -2.0, 3.0, 4.0)
        self.assertAlmostEqual(r.minx, -1.0)
        self.assertAlmostEqual(r.miny, -2.0)
        self.assertAlmostEqual(r.maxx, 3.0)
        self.assertAlmostEqual(r.maxy, 4.0)

    def testRectObjConstructorBadXCoords(self):
        """an invalid extent raises proper error (x direction)"""
        self.assertRaises(
            mapscript.MapServerError, mapscript.rectObj, 1.0, -2.0, -3.0, 4.0
        )

    def testRectObjConstructorBadYCoords(self):
        """an invalid extent raises proper error (y direction)"""
        self.assertRaises(
            mapscript.MapServerError, mapscript.rectObj, -1.0, 2.0, 3.0, -2.0
        )

    def testRectObjToPolygon(self):
        """a rect can be converted into a MS_POLYGON shape"""
        r = mapscript.rectObj(-1.0, -2.0, 3.0, 4.0)
        s = r.toPolygon()
        assert s.numlines == 1, s.numlines
        line = self.getLineFromShape(s, 0)
        assert line.numpoints == 5, line.numpoints
        point = self.getPointFromLine(line, 0)
        self.assertAlmostEqual(point.x, -1.0)
        self.assertAlmostEqual(point.y, -2.0)

    def testExceptionMessage(self):
        """test formatted error message"""
        try:
            r = mapscript.rectObj(1.0, -2.0, -3.0, 4.0)
            print(r)
        except mapscript.MapServerError as msg:
            assert (
                str(msg) == "rectObj(): Invalid rectangle. { 'minx': 1.000000 , "
                "'miny': -2.000000 , 'maxx': -3.000000 , 'maxy': 4.000000 }"
            ), msg

    def testRect__str__(self):
        """__str__ returns properly formatted string"""
        r = mapscript.rectObj(-1.0, -2.0, 3.0001, 4.0)
        r_str = str(r)
        assert (
            r_str == "{ 'minx': -1 , 'miny': -2 , 'maxx': 3.0001 , 'maxy': 4 }"
        ), r_str
        r2 = eval(r_str)
        self.assertAlmostEqual(r2["minx"], r.minx)
        self.assertAlmostEqual(r2["miny"], r.miny)
        self.assertAlmostEqual(r2["maxx"], r.maxx)
        self.assertAlmostEqual(r2["maxy"], r.maxy)

    def testRectToString(self):
        """return properly formatted string"""
        r = mapscript.rectObj(-1.0, -2.0, 3.0001, 4.0)
        r_str = r.toString()
        assert (
            r_str == "{ 'minx': -1 , 'miny': -2 , 'maxx': 3.0001 , 'maxy': 4 }"
        ), r_str

    def testRectContainsPoint(self):
        """point is contained (spatially) in rectangle"""
        r = mapscript.rectObj(-1.0, -2.0, 3.0, 4.0)
        p = mapscript.pointObj(0.0, 0.0)
        assert p in r, (p.x, p.y, r)

    def testRectContainsPointNot(self):
        """point is not contained (spatially) in rectangle"""
        r = mapscript.rectObj(-1.0, -2.0, 3.0, 4.0)
        p = mapscript.pointObj(3.00001, 0.0)
        assert p not in r, (p.x, p.y, r)


class ImageRectObjTestCase(MapPrimitivesTestCase):
    def testRectObjConstructorArgs(self):
        """create a rect in image units"""
        r = mapscript.rectObj(-1.0, 2.0, 3.0, 0.0, mapscript.MS_TRUE)
        self.assertAlmostEqual(r.minx, -1.0)
        self.assertAlmostEqual(r.miny, 2.0)
        self.assertAlmostEqual(r.maxx, 3.0)
        self.assertAlmostEqual(r.maxy, 0.0)

    def testRectObjConstructorBadYCoords(self):
        """in image units miny should be greater than maxy"""
        self.assertRaises(
            mapscript.MapServerError,
            mapscript.rectObj,
            -1.0,
            0.0,
            3.0,
            2.0,
            mapscript.MS_TRUE,
        )


if __name__ == "__main__":
    unittest.main()



# Project:  MapServer
# Purpose:  xUnit style Python mapscript tests of Layer
# Author:   Sean Gillies, sgillies@frii.com
#
# ===========================================================================
# Copyright (c) 2004, Sean Gillies
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ===========================================================================

import unittest
from random import random

import mapscript

from .testing import MapTestCase


class MapLayerTestCase(MapTestCase):
    def setUp(self):
        MapTestCase.setUp(self)
        self.layer = self.map.getLayer(1)


class LayerConstructorTestCase(MapLayerTestCase):
    def testLayerConstructorNoArg(self):
        """test layer constructor with no argument"""
        layer = mapscript.layerObj()
        assert layer.__class__.__name__ == "layerObj"
        assert layer.thisown == 1
        assert layer.index == -1
        assert layer.map is None, layer.map

    def testLayerConstructorMapArg(self):
        """test layer constructor with map argument"""
        layer = mapscript.layerObj(self.map)
        assert layer.__class__.__name__ == "layerObj"
        assert layer.thisown == 1
        lyr = self.map.getLayer(self.map.numlayers - 1)
        assert lyr is not None
        # assert str(layer) == str(lyr) # TODO - check why these are not equal
        assert layer.map is not None, layer.map


class LayerItemDefinitionTestCase(MapLayerTestCase):
    def setUp(self):
        MapTestCase.setUp(self)
        self.layer = self.map.getLayerByName("POINT")
        self.layer.open()

    def tearDown(self):
        self.layer.close()

    def testLayerGetItemType(self):
        """test getting layer item information for the first item"""

        item_type = self.layer.getItemType(0)
        assert item_type == "Integer"

    def testLayerGetItemDefinition2(self):
        """test getting layer item information for the second item"""

        item_type = self.layer.getItemType(1)
        assert item_type == "Character"

    def testLayerGetMissingItemType(self):
        """test getting item information for a non-existent index"""
        item_type = self.layer.getItemType(100)
        assert item_type is None

    def testLayerGetItemTypeClosedLayer(self):
        """item definition will be None for a closed layer"""
        self.layer.close()
        item_type = self.layer.getItemType(0)
        assert item_type is None

    def testLayerGetNonDefinedItemType(self):
        """test getting layer item information for a layer with gml_types auto"""

        layer = self.map.getLayerByName("POLYGON")
        layer.open()
        item_type = layer.getItemType(0)
        assert item_type == ""
        layer.close()


class LayerCloningTestCase(MapLayerTestCase):
    def testLayerCloning(self):
        """check attributes of a cloned layer"""
        clone = self.layer.clone()
        assert clone.thisown == 1
        assert str(clone) != str(self.layer)
        assert clone.name == self.layer.name
        assert clone.numclasses == self.layer.numclasses
        assert clone.map is None, clone.map
        assert clone.data == self.layer.data


class LayerExtentTestCase(MapTestCase):
    def setUp(self):
        MapTestCase.setUp(self)
        self.layer = self.map.getLayerByName("POLYGON")

    def testPolygonExtent(self):
        """retrieve the extent of a polygon layer"""
        e = mapscript.rectObj()
        self.assertRectsEqual(e, self.layer.extent)

    def testPolygonGetExtent(self):
        """retrieve the extent of a polygon layer"""
        e = mapscript.rectObj(-0.25, 51.227222, 0.25, 51.727222)
        self.assertRectsEqual(e, self.layer.getExtent())

    def testGetPresetExtent(self):
        """test layer.setExtent() and layer.getExtent() to return preset instead of calculating extents"""
        r = mapscript.rectObj(1.0, 1.0, 3.0, 3.0)
        self.layer.setExtent(r.minx, r.miny, r.maxx, r.maxy)
        rect = self.layer.extent
        assert r.minx == rect.minx, rect
        assert r.miny == rect.miny, rect.miny
        assert r.maxx == rect.maxx, rect.maxx
        assert r.maxy == rect.maxy, rect.maxy

    def testResetLayerExtent(self):
        """test resetting a layer's extent"""
        layer = self.map.getLayerByName("POLYGON")
        layer.setExtent()
        self.assertRectsEqual(layer.extent, mapscript.rectObj())

    def testDirectExtentAccess(self):
        """direct access to a layer's extent works properly"""
        pt_layer = self.map.getLayerByName("POINT")
        rect = pt_layer.extent
        assert str(pt_layer.extent) == str(rect), (pt_layer.extent, rect)
        e = mapscript.rectObj(-0.5, 51.0, 0.5, 52.0)
        self.assertRectsEqual(e, rect)


class LayerRasterProcessingTestCase(MapLayerTestCase):
    def testAddProcessing(self):
        """adding a layer's processing directive works"""
        self.layer.addProcessing("directive0=foo")
        assert self.layer.numprocessing == 1, self.layer.numprocessing
        self.layer.addProcessing("directive1=bar")
        assert self.layer.numprocessing == 2, self.layer.numprocessing
        directives = [
            self.layer.getProcessing(i) for i in range(self.layer.numprocessing)
        ]
        assert directives == ["directive0=foo", "directive1=bar"]

    def testClearProcessing(self):
        """clearing a self.layer's processing directive works"""
        self.layer.addProcessing("directive0=foo")
        assert self.layer.numprocessing == 1, self.layer.numprocessing
        self.layer.addProcessing("directive1=bar")
        assert self.layer.numprocessing == 2, self.layer.numprocessing
        assert self.layer.clearProcessing() == mapscript.MS_SUCCESS


class RemoveClassTestCase(MapLayerTestCase):
    def testRemoveClass1NumClasses(self):
        """RemoveClassTestCase.testRemoveClass1NumClasses: removing the layer's first class
        by index leaves one class left"""
        c = self.layer.removeClass(0)
        assert c.thisown == 1
        assert self.layer.numclasses == 1

    def testRemoveClass1ClassName(self):
        """RemoveClassTestCase.testRemoveClass1ClassName:
        confirm removing the layer's first class reverts the name of the second class"""
        c2name = self.layer.getClass(1).name
        c = self.layer.removeClass(0)
        assert c is not None
        assert self.layer.getClass(0).name == c2name

    def testRemoveClass2NumClasses(self):
        """RemoveClassTestCase.testRemoveClass2NumClasses:
        removing the layer's second class by index leaves one class left"""
        c = self.layer.removeClass(1)
        assert c is not None
        assert self.layer.numclasses == 1

    def testRemoveClass2ClassName(self):
        """RemoveClassTestCase.testRemoveClass2ClassName: confirm removing the layer's second class reverts
        the name of the first class"""
        c1name = self.layer.getClass(0).name
        c = self.layer.removeClass(1)
        assert c is not None
        assert self.layer.getClass(0).name == c1name


class InsertClassTestCase(MapLayerTestCase):
    def testLayerInsertClass(self):
        """insert class at default index"""
        n = self.layer.numclasses
        new_class = mapscript.classObj()
        new_class.name = "foo"
        new_index = self.layer.insertClass(new_class)
        assert new_index == n
        assert self.layer.numclasses == n + 1
        c = self.layer.getClass(new_index)
        assert c.thisown == 1
        assert c.name == new_class.name

    def testLayerInsertClassAtZero(self):
        """insert class at index 0"""
        n = self.layer.numclasses
        new_class = mapscript.classObj()
        new_class.name = "foo"
        new_index = self.layer.insertClass(new_class, 0)
        assert new_index == 0
        assert self.layer.numclasses == n + 1
        c = self.layer.getClass(new_index)
        assert c.thisown == 1
        assert c.name == new_class.name

    def testInsertNULLClass(self):
        """inserting NULL class should raise an error"""
        self.assertRaises(mapscript.MapServerChildError, self.layer.insertClass, None)


class LayerTestCase(MapTestCase):
    def testLayerConstructorOwnership(self):
        """LayerTestCase.testLayerConstructorOwnership: newly constructed layer has proper ownership"""
        layer = mapscript.layerObj(self.map)
        assert layer.thisown == 1

    def testGetLayerOrder(self):
        """LayerTestCase.testGetLayerOrder: get layer drawing order"""
        order = self.map.getLayerOrder()
        assert order == tuple(range(7)), order

    def testSetLayerOrder(self):
        """LayerTestCase.testSetLayerOrder: set layer drawing order"""
        ord = (1, 0, 2, 3, 4, 5, 6)
        self.map.setLayerOrder(ord)
        order = self.map.getLayerOrder()
        assert order == ord, order


# Layer removal tests


class RemoveLayerTestCase(MapTestCase):
    def testRemoveLayer1NumLayers(self):
        """removing the first layer by index from the mapfile leaves three"""
        self.map.removeLayer(0)
        assert self.map.numlayers == 6

    def testRemoveLayer1LayerName(self):
        """removing first layer reverts it to the second layer's name"""
        l2name = self.map.getLayer(1).name
        self.map.removeLayer(0)
        assert self.map.getLayer(0).name == l2name

    def testRemoveLayer2NumLayers(self):
        """removing second layer by index from mapfile leaves three layers"""
        self.map.removeLayer(1)
        assert self.map.numlayers == 6

    def testRemoveLayer2LayerName(self):
        """removing of the second layer reverts it to the first layer's name"""
        l1name = self.map.getLayer(0).name
        self.map.removeLayer(1)
        assert self.map.getLayer(0).name == l1name


class ExpressionTestCase(MapLayerTestCase):
    def testClearExpression(self):
        """layer expression can be properly cleared"""
        self.layer.setFilter("")
        fs = self.layer.getFilterString()
        assert fs is None, fs

    def testSetStringExpression(self):
        """layer expression can be set to string"""
        self.layer.setFilter("foo")
        fs = self.layer.getFilterString()
        self.layer.filteritem = "fid"
        assert fs == '"foo"', fs
        self.map.draw()

    def testSetQuotedStringExpression(self):
        """layer expression string can be quoted"""
        self.layer.setFilter('"foo"')
        fs = self.layer.getFilterString()
        self.layer.filteritem = "fid"
        assert fs == '"foo"', fs
        self.map.draw()

    def testSetRegularExpression(self):
        """layer expression can be regular expression"""
        self.layer.setFilter("/foo/")
        self.layer.filteritem = "fid"
        fs = self.layer.getFilterString()
        assert fs == "/foo/", fs
        self.map.draw()

    def testSetLogicalExpression(self):
        """layer expression can be logical expression"""
        self.layer.setFilter("([fid] >= 2)")
        fs = self.layer.getFilterString()
        assert fs == "([fid] >= 2)", fs
        self.map.draw()

    def testSetCompoundLogicalExpression(self):
        """layer expression can be a compound logical expression"""
        # filter = '( ([fid] >= 2) AND (\'[fname]\' == \'A Polygon\' ))'
        flt = """( ([fid] >= 2) AND ("[fname]" == 'A Polygon' ))"""
        excode = self.layer.setFilter(flt)
        assert excode == mapscript.MS_SUCCESS, excode
        fs = self.layer.getFilterString()
        assert fs == flt, fs
        self.map.draw()


class LayerQueryTestCase(MapLayerTestCase):
    def setUp(self):
        MapLayerTestCase.setUp(self)
        self.layer = self.map.getLayerByName("POINT")
        self.layer.template = "foo"


class SpatialLayerQueryTestCase(LayerQueryTestCase):
    def testRectQuery(self):
        qrect = mapscript.rectObj(-10.0, 45.0, 10.0, 55.0)
        self.layer.queryByRect(self.map, qrect)
        assert self.layer.getNumResults() == 1

    def testShapeQuery(self):
        qrect = mapscript.rectObj(-10.0, 45.0, 10.0, 55.0)
        qshape = qrect.toPolygon()
        self.layer.queryByShape(self.map, qshape)
        assert self.layer.getNumResults() == 1

    def testPointQuery(self):
        qpoint = mapscript.pointObj(0.0, 51.5)
        self.layer.queryByPoint(self.map, qpoint, mapscript.MS_MULTIPLE, 2.0)
        assert self.layer.getNumResults() == 1

    def testRectQueryNoResults(self):
        qrect = mapscript.rectObj(-101.0, 0.0, -100.0, 1.0)
        self.layer.queryByRect(self.map, qrect)
        assert self.layer.getNumResults() == 0

    def testShapeQueryNoResults(self):
        qrect = mapscript.rectObj(-101.0, 0.0, -100.0, 1.0)
        qshape = qrect.toPolygon()
        self.layer.queryByShape(self.map, qshape)
        assert self.layer.getNumResults() == 0

    def testPointQueryNoResults(self):
        qpoint = mapscript.pointObj(-100.0, 51.5)
        self.layer.queryByPoint(self.map, qpoint, mapscript.MS_MULTIPLE, 2.0)
        assert self.layer.getNumResults() == 0


class AttributeLayerQueryTestCase(LayerQueryTestCase):
    def testAttributeQuery(self):
        self.layer.queryByAttributes(
            self.map, "FNAME", '"A Point"', mapscript.MS_MULTIPLE
        )
        assert self.layer.getNumResults() == 1

    def testLogicalAttributeQuery(self):
        self.layer.queryByAttributes(
            self.map, None, '("[FNAME]" == "A Point")', mapscript.MS_MULTIPLE
        )
        assert self.layer.getNumResults() == 1

    def testAttributeQueryNoResults(self):
        self.layer.queryByAttributes(
            self.map, "FNAME", '"Bogus Point"', mapscript.MS_MULTIPLE
        )
        assert self.layer.getNumResults() == 0


class LayerVisibilityTestCase(MapLayerTestCase):
    def setUp(self):
        MapLayerTestCase.setUp(self)
        self.layer.minscaledenom = 1000
        self.layer.maxscaledenom = 2000
        self.layer.status = mapscript.MS_ON
        self.map.zoomScale(
            1500, mapscript.pointObj(100, 100), 200, 200, self.map.extent, None
        )

    def testInitialVisibility(self):
        """expect visibility"""
        assert self.layer.isVisible() == mapscript.MS_TRUE

    def testStatusOffVisibility(self):
        """expect false visibility after switching status off"""
        self.layer.status = mapscript.MS_OFF
        assert self.layer.isVisible() == mapscript.MS_FALSE

    def testZoomOutVisibility(self):
        """expect false visibility after zooming out beyond maximum"""
        self.map.zoomScale(
            2500, mapscript.pointObj(100, 100), 200, 200, self.map.extent, None
        )
        assert self.layer.isVisible() == mapscript.MS_FALSE


class InlineLayerTestCase(unittest.TestCase):
    def setUp(self):
        # Inline feature layer
        self.ilayer = mapscript.layerObj()
        self.ilayer.type = mapscript.MS_LAYER_POLYGON
        self.ilayer.status = mapscript.MS_DEFAULT
        self.ilayer.connectiontype = mapscript.MS_INLINE

        cs = "f7fcfd, e5f5f9, ccece6, 99d8c9, 66c2a4, 41ae76, 238b45, 006d2c, 00441b"
        colors = ["#" + h for h in cs.split(", ")]

        for i in range(9):
            # Make a class for feature
            ci = self.ilayer.insertClass(mapscript.classObj())
            co = self.ilayer.getClass(ci)
            si = co.insertStyle(mapscript.styleObj())
            so = co.getStyle(si)
            so.color.setHex(colors[i])
            li = co.addLabel(mapscript.labelObj())
            lbl = co.getLabel(li)
            lbl.color.setHex("#000000")
            lbl.outlinecolor.setHex("#FFFFFF")
            lbl.type = mapscript.MS_BITMAP
            lbl.size = mapscript.MS_SMALL

            # The shape to add is randomly generated
            xc = 4.0 * (random() - 0.5)
            yc = 4.0 * (random() - 0.5)
            r = mapscript.rectObj(xc - 0.25, yc - 0.25, xc + 0.25, yc + 0.25)
            s = r.toPolygon()

            # Classify
            s.classindex = i
            s.text = "F%d" % (i)

            # Add to inline feature layer
            self.ilayer.addFeature(s)

    def testExternalClassification(self):
        mo = mapscript.mapObj()
        mo.setSize(400, 400)
        mo.setExtent(-2.5, -2.5, 2.5, 2.5)
        mo.selectOutputFormat("image/png")
        mo.insertLayer(self.ilayer)
        im = mo.draw()
        im.save("testExternalClassification.png")


if __name__ == "__main__":
    unittest.main()



# Project:  MapServer
# Purpose:  xUnit style Python mapscript tests of SymbolSet
# Author:   Sean Gillies, sgillies@frii.com
#
# ===========================================================================
# Copyright (c) 2004, Sean Gillies
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ===========================================================================

import os
import unittest

import mapscript

from .testing import TESTS_PATH, XMARKS_IMAGE, MapTestCase

SYMBOLSET = os.path.join(TESTS_PATH, "symbols.txt")


class SymbolSetTestCase(unittest.TestCase):
    def testConstructorNoArgs(self):
        """new instance of symbolSetObj should have one symbol"""
        symbolset = mapscript.symbolSetObj()
        num = symbolset.numsymbols
        assert num == 1, num

    def testConstructorFile(self):
        """new instance of symbolSetObj from symbols.txt"""
        symbolset = mapscript.symbolSetObj(SYMBOLSET)
        num = symbolset.numsymbols
        assert num == 4, num

    def testAddSymbolToNewSymbolSet(self):
        """add two new symbols to a SymbolSet"""
        symbolset = mapscript.symbolSetObj(SYMBOLSET)
        symbola = mapscript.symbolObj("testa")
        symbolb = mapscript.symbolObj("testb")
        symbolset.appendSymbol(symbola)
        symbolset.appendSymbol(symbolb)
        num = symbolset.numsymbols
        assert num == 6, num
        names = [None, "circle", "xmarks-png", "home-png", "testa", "testb"]
        for i in range(symbolset.numsymbols):
            symbol = symbolset.getSymbol(i)
            assert symbol.name == names[i], symbol.name

    def testRemoveSymbolFromNewSymbolSet(self):
        """after removing a symbol, expect numsymbols -= 1"""
        symbolset = mapscript.symbolSetObj(SYMBOLSET)
        symbolset.removeSymbol(1)
        num = symbolset.numsymbols
        assert num == 3, num

    def testSaveNewSymbolSet(self):
        """save a new SymbolSet to disk"""
        symbolset = mapscript.symbolSetObj(SYMBOLSET)
        symbola = mapscript.symbolObj("testa")
        symbolb = mapscript.symbolObj("testb")
        symbolset.appendSymbol(symbola)
        symbolset.appendSymbol(symbolb)
        assert symbolset.save("new_symbols.txt") == mapscript.MS_SUCCESS

    def testError(self):
        symbolset = mapscript.symbolSetObj(SYMBOLSET)
        symbola = mapscript.symbolObj("testa")
        symbolb = mapscript.symbolObj("testb")
        symbolset.appendSymbol(symbola)
        symbolset.appendSymbol(symbolb)
        self.assertRaises(
            mapscript.MapServerError, symbolset.save, "/bogus/new_symbols.txt"
        )


class MapSymbolSetTestCase(MapTestCase):
    def testGetNumSymbols(self):
        """expect getNumSymbols == 2 from test fixture test.map"""
        num = self.map.getNumSymbols()
        assert num == 4, num

    def testSymbolSetNumsymbols(self):
        """expect numsymbols == 2 from test fixture test.map"""
        num = self.map.symbolset.numsymbols
        assert num == 4, num

    def testSymbolSetSymbolNames(self):
        """test names of symbols in test fixture's symbolset"""
        set = self.map.symbolset
        names = [None, "circle", "xmarks-png", "home-png"]
        for i in range(set.numsymbols):
            symbol = set.getSymbol(i)
            assert symbol.name == names[i], symbol.name

    def testSymbolIndex(self):
        """expect index of 'circle' symbol to be 1 in test fixture symbolset"""
        i = self.map.symbolset.index("circle")
        assert i == 1, i

    def testBug1962(self):
        """resetting imagepath doesn't cause segfault"""
        layer = self.map.getLayerByName("POINT")
        style0 = layer.getClass(0).getStyle(0)
        sym0 = style0.symbol
        sym1 = self.map.symbolset.getSymbol(sym0)
        sym2 = mapscript.symbolObj("xxx")
        assert sym2 is not None
        sym1.setImagepath(XMARKS_IMAGE)
        # self.assertRaises(IOError, sym1.setImagepath, '/bogus/new_symbols.txt')

        msimg = self.map.draw()
        assert msimg.thisown == 1
        data = msimg.getBytes()
        filename = "testBug1962.png"
        fh = open(filename, "wb")
        fh.write(data)
        fh.close()

    def testDrawNewSymbol(self):
        """draw using a new symbol added to the fixture"""
        symbol = mapscript.symbolObj("xmarks", XMARKS_IMAGE)
        symbol_index = self.map.symbolset.appendSymbol(symbol)
        assert symbol_index == 4, symbol_index
        num = self.map.symbolset.numsymbols
        assert num == 5, num
        inline_layer = self.map.getLayerByName("INLINE")
        s = inline_layer.getClass(0).getStyle(0)
        s.symbol = symbol_index
        # s.size = 24
        msimg = self.map.draw()
        assert msimg.thisown == 1
        filename = "testDrawNewSymbol.png"
        fh = open(filename, "wb")
        msimg.write(fh)
        fh.close()


if __name__ == "__main__":
    unittest.main()



# Project:  MapServer
# Purpose:  xUnit style Python mapscript tests of ResultCache
# Author:   Sean Gillies, sgillies@frii.com
#
# ===========================================================================
# Copyright (c) 2004, Sean Gillies
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ===========================================================================

import unittest

import mapscript

from .testing import MapTestCase


class LayerQueryTestCase(MapTestCase):
    def setUp(self):
        MapTestCase.setUp(self)
        self.layer = self.map.getLayer(1)
        self.layer.template = "some day i will fix this!"

    def pointquery(self):
        p = mapscript.pointObj(0.0, 51.5)
        self.layer.queryByPoint(self.map, p, mapscript.MS_MULTIPLE, -1)
        return self.layer.getResults()

    def indexquery(self):
        self.layer.queryByIndex(self.map, -1, 0, mapscript.MS_TRUE)
        return self.layer.getResults()


class ResultCacheTestCase(LayerQueryTestCase):
    def testNoDirectAccess(self):
        """denying direct access to layer's resultcache"""
        self.assertRaises(AttributeError, getattr, self.layer, "resultcache")

    def testNullResultCache(self):
        """before a query layer's resultcache should be NULL"""
        results = self.layer.getResults()
        assert results is None


class PointQueryResultsTestCase(LayerQueryTestCase):
    def testCacheAfterQuery(self):
        """simple point query returns one result"""
        results = self.pointquery()
        assert results.numresults == 1

    def testCacheAfterFailedQuery(self):
        """point query with no results returns NULL"""
        p = mapscript.pointObj(0.0, 0.0)
        self.layer.queryByPoint(self.map, p, mapscript.MS_MULTIPLE, -1)
        results = self.layer.getResults()
        assert results is None

    def testDeletionOfResults(self):
        """deleting results should not harm the layer"""
        results = self.pointquery()
        assert results.thisown == 0
        del results
        results = self.layer.getResults()
        assert results.numresults == 1

    def testQueryResultBounds(self):
        """result bounds should equal layer bounds"""
        results = self.pointquery()
        e = self.layer.getExtent()
        self.assertRectsEqual(results.bounds, e)

    def testQueryResultMembers(self):
        """get the single result member"""
        results = self.pointquery()
        self.layer.open()
        res = results.getResult(0)
        feature = self.layer.getShape(res)
        self.layer.close()
        self.assertRectsEqual(results.bounds, feature.bounds)
        assert feature.getValue(1) == "A Polygon"

    def testQueryResultsListComp(self):
        """get all results using list comprehension"""
        results = self.pointquery()
        result_list = [results.getResult(i) for i in range(results.numresults)]
        assert len(result_list) == 1
        assert result_list[0].shapeindex == 0

    def testQueryByIndex(self):
        """pop a result into the result set"""
        self.layer.queryByIndex(self.map, -1, 0, mapscript.MS_FALSE)
        results = self.layer.getResults()
        assert results.numresults == 1
        self.layer.queryByIndex(self.map, -1, 0, mapscript.MS_TRUE)
        results = self.layer.getResults()
        assert results.numresults == 2


class DumpAndLoadTestCase(LayerQueryTestCase):
    def testSaveAndLoadQuery(self):
        """test saving query to a file"""
        results = self.pointquery()
        self.map.saveQuery("test.qy")
        self.map.freeQuery()
        results = self.layer.getResults()
        assert results is None
        self.map.loadQuery("test.qy")
        results = self.layer.getResults()
        assert results is not None


class LayerOffQueryTestCase(LayerQueryTestCase):
    def testPointQueryOffLayer(self):
        """simple point query returns one result, even if layer is off"""
        self.layer.status = mapscript.MS_OFF
        results = self.pointquery()
        assert results.numresults == 1

    def testIndexQueryOffLayer(self):
        """simple index query returns one result, even if layer is off"""
        self.layer.status = mapscript.MS_OFF
        results = self.indexquery()
        assert results.numresults == 1


if __name__ == "__main__":
    unittest.main()



# Project:  MapServer
# Purpose:  xUnit style Python mapscript tests of hashTableObj
# Author:   Sean Gillies, sgillies@frii.com
#
# ===========================================================================
# Copyright (c) 2004, Sean Gillies
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ===========================================================================

import os
import unittest

import mapscript

from .testing import TESTS_PATH


class FontTestCase(unittest.TestCase):
    def testSettingFonts(self):
        mo = mapscript.mapObj()
        assert mo.fontset.numfonts == 0
        mo.fontset.fonts.set("Vera", os.path.join(TESTS_PATH, "vera", "Vera.ttf"))
        # NB: this does *not* increment the fonset.numfonts -- new bug

        mo.setSize(300, 300)
        mo.setExtent(-1.0, -1.0, 1.0, 1.0)

        lo = mapscript.layerObj()
        lo.type = mapscript.MS_LAYER_POINT
        lo.connectiontype = mapscript.MS_INLINE
        lo.status = mapscript.MS_DEFAULT

        co = mapscript.classObj()
        lbl = mapscript.labelObj()
        lbl.type = mapscript.MS_TRUETYPE
        lbl.font = "Vera"
        lbl.size = 10
        lbl.color.setHex("#000000")
        co.addLabel(lbl)

        so = mapscript.styleObj()
        so.symbol = 0
        so.color.setHex("#000000")

        co.insertStyle(so)
        lo.insertClass(co)
        li = mo.insertLayer(lo)
        lo = mo.getLayer(li)

        point = mapscript.pointObj(0, 0)
        line = mapscript.lineObj()
        line.add(point)
        shape = mapscript.shapeObj(lo.type)
        shape.add(line)
        shape.setBounds()
        shape.text = "Foo"
        shape.classindex = 0

        lo.addFeature(shape)
        im = mo.draw()

        # im = mo.prepareImage()
        # shape.draw(mo, lo, im)
        im.save("testSettingFonts.png")


if __name__ == "__main__":
    unittest.main()



# Project:  MapServer
# Purpose:  xUnit style Python mapscript tests of Map "zooming"
# Author:   Sean Gillies, sgillies@frii.com
#
# ===========================================================================
# Copyright (c) 2004, Sean Gillies
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ===========================================================================


import unittest

import mapscript

from .testing import MapPrimitivesTestCase


class LineObjTestCase(MapPrimitivesTestCase):
    """Testing the lineObj class in stand-alone mode"""

    def setUp(self):
        """The test fixture is a line with two points"""
        self.points = (mapscript.pointObj(0.0, 1.0), mapscript.pointObj(2.0, 3.0))
        self.line = mapscript.lineObj()
        self.addPointToLine(self.line, self.points[0])
        self.addPointToLine(self.line, self.points[1])

    def testCreateLine(self):
        """created line has the correct number of points"""
        assert self.line.numpoints == 2

    def testGetPointsFromLine(self):
        """points in the line share are the same that were input"""
        for i in range(len(self.points)):
            got_point = self.getPointFromLine(self.line, i)
            self.assertPointsEqual(got_point, self.points[i])

    def testAddPointToLine(self):
        """adding a point to a line behaves correctly"""
        new_point = mapscript.pointObj(4.0, 5.0)
        self.addPointToLine(self.line, new_point)
        assert self.line.numpoints == 3

    def xtestAlterNumPoints(self):
        """
        numpoints is immutable, this should raise error
        """
        # The following works locally but fails on Appveyor (error is not raised)
        # with self.assertRaises(AttributeError):
        #    self.line.numpoints = 3

        # the following approach also fails on appveyor
        error_thrown = False
        try:
            self.line.numpoints = 3
        except AttributeError:
            error_thrown = True

        assert error_thrown is True

    def testLineGeoInterface(self):
        """return line using the  __geo_interface__ protocol"""
        assert self.line.__geo_interface__ == {
            "type": "LineString",
            "coordinates": [(0.0, 1.0, 0.0), (2.0, 3.0, 0.0)],
        }


if __name__ == "__main__":
    unittest.main()



# Project:  MapServer
# Purpose:  xUnit style Python mapscript tests of map cloning
# Author:   Sean Gillies, sgillies@frii.com
#
# ===========================================================================
# Copyright (c) 2004, Sean Gillies
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ===========================================================================

import unittest

import mapscript

from .testing import TESTMAPFILE


class MapCloneTestCase(unittest.TestCase):
    """Base class for testing with a map fixture"""

    def setUp(self):
        self.mapobj_orig = mapscript.mapObj(TESTMAPFILE)
        self.mapobj_clone = self.mapobj_orig.clone()

    def tearDown(self):
        self.mapobj_orig = None
        self.mapobj_clone = None


class MapCloningTestCase(MapCloneTestCase):
    def testClonedName(self):
        """MapCloningTestCase.testClonedName: the name of a cloned map equals the original"""
        assert self.mapobj_clone.name == self.mapobj_orig.name

    def testClonedLayers(self):
        """MapCloningTestCase.testClonedLayers: the layers of a cloned map equal the original"""
        assert self.mapobj_clone.numlayers == self.mapobj_orig.numlayers
        assert self.mapobj_clone.getLayer(0).data == self.mapobj_orig.getLayer(0).data

    def testSetFontSet(self):
        """MapCloningTestCase.testSetFontSet: the number of fonts in a cloned map equal original"""
        assert self.mapobj_clone.fontset.numfonts == 2

    def testSetSymbolSet(self):
        """MapCloningTestCase.testSetSymbolSet: the number of symbols in a cloned map equal original"""
        num = self.mapobj_clone.symbolset.numsymbols
        assert num == 5, num

    def testDrawClone(self):
        """drawing a cloned map works properly"""
        msimg = self.mapobj_clone.draw()
        assert msimg.thisown == 1
        filename = "testClone.png"
        fh = open(filename, "wb")
        msimg.write(fh)
        fh.close()

    def testDrawCloneJPEG(self):
        """drawing a cloned map works properly"""
        self.mapobj_clone.selectOutputFormat("JPEG")
        self.mapobj_clone.getLayerByName("INLINE-PIXMAP-RGBA").status = mapscript.MS_ON
        msimg = self.mapobj_clone.draw()
        assert msimg.thisown == 1
        filename = "testClone.jpg"
        fh = open(filename, "wb")
        msimg.write(fh)
        fh.close()

    def testDrawClonePNG24(self):
        """drawing a cloned map works properly"""
        self.mapobj_clone.selectOutputFormat("PNG24")
        self.mapobj_clone.getLayerByName("INLINE-PIXMAP-RGBA").status = mapscript.MS_ON
        msimg = self.mapobj_clone.draw()
        assert msimg.thisown == 1
        filename = "testClonePNG24.png"
        fh = open(filename, "wb")
        msimg.write(fh)
        fh.close()


if __name__ == "__main__":
    unittest.main()



# Project:  MapServer
# Purpose:  xUnit style Python mapscript tests of hashTableObj
# Author:   Sean Gillies, sgillies@frii.com
#
# ===========================================================================
# Copyright (c) 2004, Sean Gillies
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.
# ===========================================================================

import unittest

import mapscript

from .testing import MapTestCase

# ===========================================================================
# Base class for hashtable tests.  Derived classes use these tests, but
# defined different values for self.table.


class HashTableBaseTestCase(object):

    keys = ["key1", "key2", "key3", "key4"]
    values = ["value1", "value2", "value3", "value4"]

    def testUseNonExistentKey(self):
        assert self.table.get("bogus") is None

    def testUseNonExistentKeyWithDefault(self):
        assert self.table.get("bogus", "default") == "default"

    def testGetValue(self):
        for key, value in zip(self.keys, self.values):
            assert self.table.get(key) == value
            assert self.table.get(key.upper()) == value
            assert self.table.get(key.capitalize()) == value

    def testGetValueWithDefault(self):
        for key, value in zip(self.keys, self.values):
            assert self.table.get(key, "foo") == value
            assert self.table.get(key.upper(), "foo") == value
            assert self.table.get(key.capitalize(), "foo") == value

    def testClear(self):
        self.table.clear()
        for key in self.keys:
            assert self.table.get(key) is None

    def testRemoveItem(self):
        key = self.keys[0]
        self.table.remove(key)
        assert self.table.get(key) is None

    def testNextKey(self):
        key = self.table.nextKey()
        assert key == self.keys[0], key
        for i in range(1, len(self.keys)):
            key = self.table.nextKey(key)
            assert key == self.keys[i], key
        # We're at the end, next should be None
        key = self.table.nextKey(key)
        assert key is None, key

    # tests using the Python dictionary access methods

    def testDictKeys(self):
        assert sorted(self.keys) == sorted(self.table.keys())

    def testDictItems(self):
        assert len(self.keys) == len(self.table)

    def testCheckDictContains(self):
        for key in self.keys:
            assert key in self.table.keys()

    def testGetDictValue(self):
        for key, value in zip(self.keys, self.values):
            assert self.table[key] == value
            assert self.table[key.upper()] == value
            assert self.table[key.capitalize()] == value

    def testRemoveDictItem(self):
        key = self.keys[0]
        del self.table[key]
        assert self.table[key] is None


# ===========================================================================
# Test begins now

# ===========================================================================
# HashTable test case
#


class HashTableTestCase(unittest.TestCase, HashTableBaseTestCase):
    """Tests of the MapServer HashTable object"""

    def setUp(self):
        "our fixture is a HashTable with two items"
        self.table = mapscript.hashTableObj()
        for key, value in zip(self.keys, self.values):
            self.table.set(key, value)

    def tearDown(self):
        self.table = None

    def testConstructor(self):
        table = mapscript.hashTableObj()
        assert table.__class__.__name__ == "hashTableObj", table


# ==============================================================================
# following tests use the tests/test.map fixture


class WebMetadataTestCase(MapTestCase, HashTableBaseTestCase):
    def setUp(self):
        MapTestCase.setUp(self)
        self.table = self.map.web.metadata
        self.keys = ["key1", "key2", "key3", "key4", "ows_enable_request"]
        self.values = ["value1", "value2", "value3", "value4", "*"]

    def tearDown(self):
        MapTestCase.tearDown(self)
        self.table = None


class LayerMetadataTestCase(WebMetadataTestCase):
    def setUp(self):
        MapTestCase.setUp(self)
        self.table = self.map.getLayer(1).metadata


class ClassMetadataTestCase(WebMetadataTestCase):
    def setUp(self):
        MapTestCase.setUp(self)
        self.table = self.map.getLayer(1).getClass(0).metadata


if __name__ == "__main__":
    unittest.main()
